package com.bgy.sms.channel.api;

import com.alibaba.fastjson.JSONObject;
import com.bgy.sms.channel.dmHub.config.DmHubConfig;
import com.bgy.sms.channel.dto.*;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.service.MessageService;
import com.bgy.sms.service.XiaoShuService;
import com.bgy.util.Md5Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@SuppressWarnings("Duplicates")
@RestController
@RequestMapping()
public class DmHubApi {

    private static final Logger log = LoggerFactory.getLogger(DmHubApi.class);

    @Autowired
    private MessageService messageService;
    @Autowired
    private XiaoShuService xiaoShuService;

    private static final String REJECTED = "rejected";    //  创建模板响应数据
    private static final String APPROVED = "approved";    //  创建模板响应数据

    @GetMapping("/ping")
    public String ping() {
        log.info("====sms ping=====");
        return "pong";
    }


    /**
     * 创建短信模板
     *
     * @return
     */
    @PostMapping("/sms/template")
    public String template(@RequestBody DmHubTemplateRequest params, String appId, String timestamp, String signature) {

        log.info("**********创建模板接口入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(params), appId, timestamp, signature);
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                log.info("**********创建模板接口出参*******：{}", "接口请求签名校验不通过");
                return JSONObject.toJSONString(new DmHubResponse("999", "接口请求签名校验不通过"));
            }
            DmHubResponse dmHubResponse = messageService.createTemplate(params);
            if (ResponseCode.SUCCESS.getCode().equals(dmHubResponse.getCode())) {
                String retJson = JSONObject.toJSONString(new TemplateResponse());
                log.info("返回内容：{}", retJson);
                return retJson;
            } else {
                return JSONObject.toJSONString(dmHubResponse);
            }
        } catch (Exception e) {
            log.error("创建模板短信异常", e);
            return JSONObject.toJSONString(new DmHubResponse("999", "创建模板短信异常"));
        }
    }

    /**
     * 发送短信验证码
     *
     * @return
     */
    @PostMapping("/sms/verifyCode")
    public DmHubResponse verifyCode(@RequestBody DmHubCodeRequest params, String appId, String timestamp, String signature) {
        log.info("**********发送短信验证码*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(params), appId, timestamp, signature);
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "接口请求签名校验不通过");
            }
            response = messageService.sendCode(params);
        } catch (Exception e) {
            log.error("发送验证码短信异常", e);
            response = new DmHubResponse("999", "发送验证码短信异常");
        }
        log.info("**********发送短信验证码出参*******：{}", JSONObject.toJSONString(response));
        return response;
    }


    /**
     * 批量发送（通知或营销类）
     *
     * @return
     */
    @PostMapping("/sms/batch")
    public DmHubResponse batch(@RequestBody DmHubBatchSendRequest request, String appId, String timestamp, String signature) {
        log.info("**********批量发送入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(request), appId, timestamp, signature);
        DmHubResponse response;
        long start = System.currentTimeMillis();
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "接口请求签名校验不通过");
            }
            response = messageService.batchSendBatch(request);
        } catch (Exception e) {
            log.error("发送批量短信异常", e);
            response = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        long end = System.currentTimeMillis();
        log.info("**********批量发送出参*******：{}", JSONObject.toJSONString(response));
        log.info("**********批量发送耗时*******：{}ms", end - start);
        return response;
    }

    public boolean checkSignature(String timestamp, String signature) {
        String signStr = DmHubConfig.appId + DmHubConfig.appSecret + timestamp;
        String encrypt = Md5Util.encrypt(signStr);
        log.info("待签名串：" + encrypt);
        if (!encrypt.equals(signature)) {
            log.error("签名不一致，插件生成签名：{}", encrypt);
            return false;
        }
        return true;

    }

    /**
     * 发送单条（通知或营销类）
     *
     * @return
     */
    @RequestMapping("/sms/send")
    public DmHubResponse send(@RequestBody DmHubSendRequest request, String appId, String timestamp, String signature) {
        log.info("**********单条发送入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(request), appId, timestamp, signature);
        long start = System.currentTimeMillis();
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "签名校验不通过");
            }
            response = messageService.send(request);
        } catch (Exception e) {
            log.error("发送单条短信异常", e);
            response = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        long end = System.currentTimeMillis();
        log.info("**********单条发送出参*******：{}", JSONObject.toJSONString(response));
        log.info("**********单条发送耗时*******：{}ms", end - start);
        return response;
    }

}
